#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

template<int mod>
class Modular {
public:
    int val;
    Modular() : val(0) {}
    Modular(int new_val) : val(new_val) {}
    // Modular(long long new_val) : val(new_val % mod) {}  // AFFECTS OPERATOR* (because it has one more %)
    friend Modular operator+(const Modular& a, const Modular& b) {
        if (a.val + b.val >= mod) return a.val + b.val - mod;
        else return a.val + b.val;
    }
    friend Modular operator-(const Modular& a, const Modular& b) {
        if (a.val - b.val < 0) return a.val - b.val + mod;
        else return a.val - b.val;
    }
    friend Modular operator*(const Modular& a, const Modular& b) {
        return 1ll * a.val * b.val % mod;
    }
    friend Modular binpow(Modular a, long long n) {
        Modular res = 1;
        for (; n; n >>= 1) {
            if (n & 1) res *= a;
            a *= a;
        }
        return res;
    }
    /* ALTERNATIVE INVERSE FUNCTION USING EXTENDED EUCLIDEAN ALGORITHM
    friend void gcd(int a, int b, Modular& x, Modular& y) {
        if (a == 0) {
            x = Modular(0);
            y = Modular(1);
            return;
        }
        Modular x1, y1;
        gcd(b % a, a, x1, y1);
        x = y1 - (b / a) * x1;
        y = x1;
    }
    friend Modular inv(const Modular& a) {
        Modular x, y;
        gcd(a.val, mod, x, y);
        return x;
    }
    */
    friend Modular inv(const Modular& a) {
        return binpow(a, mod - 2);
    }
    Modular operator/(const Modular& ot) const {
        return *this * inv(ot);
    }
    Modular& operator++() {
        if (val + 1 == mod) val = 0;
        else ++val;
        return *this;
    }
    Modular operator++(int) {
        Modular tmp = *this;
        ++(*this);
        return tmp;
    }
    Modular operator+() const {
        return *this;
    }
    Modular operator-() const {
        return 0 - *this;
    }
    Modular& operator+=(const Modular& ot) {
        return *this = *this + ot;
    }
    Modular& operator-=(const Modular& ot) {
        return *this = *this - ot;
    }
    Modular& operator*=(const Modular& ot) {
        return *this = *this * ot;
    }
    Modular& operator/=(const Modular& ot) {
        return *this = *this / ot;
    }
    bool operator==(const Modular& ot) const {
        return val == ot.val;
    }
    bool operator!=(const Modular& ot) const {
        return val != ot.val;
    }
    bool operator<(const Modular& ot) const {
        return val < ot.val;
    }
    bool operator>(const Modular& ot) const {
        return val > ot.val;
    }
    explicit operator int() const {
        return val;
    }
};

template<int mod>
istream& operator>>(istream& istr, Modular<mod>& x) {
    return istr >> x.val;
}

template<int mod>
ostream& operator<<(ostream& ostr, const Modular<mod>& x) {
    return ostr << x.val;
}

const int mod = 998244353; // 998244353
using Mint = Modular<mod>;

const int N = 1e6 + 10;
Mint fact[N], inv_fact[N], pw2[N], pw3[N];

void init() {
    fact[0] = pw2[0] = pw3[0] = 1;
    for (int i = 1; i <= N - 1; i++) {
        fact[i] = fact[i - 1] * i;
        pw2[i] = pw2[i - 1] + pw2[i - 1];
        pw3[i] = pw3[i - 1] + pw3[i - 1] + pw3[i - 1];
    }
    inv_fact[N - 1] = inv(fact[N - 1]);
    for (int i = N - 2; i >= 0; i--) {
        inv_fact[i] = inv_fact[i + 1] * (i + 1);
    }
}

Mint C(int n, int k) {
    if (n < 0 || k < 0 || n < k) return 0;
    return fact[n] * inv_fact[k] * inv_fact[n - k];
}

struct Matrix {

};

typedef long double ld;
const ld pi = acos(-1);
const ll mult_const = 1e9;

struct Point {
    int64_t x, y;

    Point(int64_t x_, int64_t y_): x(x_), y(y_) {}

    Point operator - (const Point & p) const {
        return Point(x - p.x, y - p.y);
    }

    int64_t cross (const Point & p) const {
        return x * p.y - y * p.x;
    }

    int64_t cross (const Point & p, const Point & q) const {
        return (p - *this).cross(q - *this);
    }

    int half () const {
        return int(y < 0 || (y == 0 && x < 0));
    }
};

std::vector<std::vector<size_t>> find_faces(std::vector<Point> vertices, std::vector<std::vector<size_t>> adj) {
    size_t n = vertices.size();
    std::vector<std::vector<char>> used(n);
    for (size_t i = 0; i < n; i++) {
        used[i].resize(adj[i].size());
        used[i].assign(adj[i].size(), 0);
        auto compare = [&](size_t l, size_t r) {
            Point pl = vertices[l] - vertices[i];
            Point pr = vertices[r] - vertices[i];
            if (pl.half() != pr.half())
                return pl.half() < pr.half();
            return pl.cross(pr) > 0;
        };
        std::sort(adj[i].begin(), adj[i].end(), compare);
    }
    std::vector<std::vector<size_t>> faces;
    for (size_t i = 0; i < n; i++) {
        for (size_t edge_id = 0; edge_id < adj[i].size(); edge_id++) {
            if (used[i][edge_id]) {
                continue;
            }
            std::vector<size_t> face;
            size_t v = i;
            size_t e = edge_id;
            while (!used[v][e]) {
                used[v][e] = true;
                face.push_back(v);
                size_t u = adj[v][e];
                size_t e1 = std::lower_bound(adj[u].begin(), adj[u].end(), v, [&](size_t l, size_t r) {
                    Point pl = vertices[l] - vertices[u];
                    Point pr = vertices[r] - vertices[u];
                    if (pl.half() != pr.half())
                        return pl.half() < pr.half();
                    return pl.cross(pr) > 0;
                }) - adj[u].begin() + 1;
                if (e1 == adj[u].size()) {
                    e1 = 0;
                }
                v = u;
                e = e1;
            }
            std::reverse(face.begin(), face.end());
            int sign = 0;
            for (size_t j = 0; j < face.size(); j++) {
                size_t j1 = (j + 1) % face.size();
                size_t j2 = (j + 2) % face.size();
                int64_t val = vertices[face[j]].cross(vertices[face[j1]], vertices[face[j2]]);
                if (val > 0) {
                    sign = 1;
                    break;
                } else if (val < 0) {
                    sign = -1;
                    break;
                }
            }
            if (sign <= 0) {
                faces.insert(faces.begin(), face);
            } else {
                faces.emplace_back(face);
            }
        }
    }
    return faces;
}

void foo() {
    vector<Point> vertices;
    vertices.emplace_back(1, 0);
    vertices.emplace_back(0, 1);
    vertices.emplace_back(-1, 0);
    vertices.emplace_back(0, -1);
    vector<vector<size_t>> adj(vertices.size());
    for (int i = 0; i < vertices.size(); i++) {
        int j = i + 1 == vertices.size() ? 0 : i + 1;
        adj[i].push_back(j);
        adj[j].push_back(i);
    }
    auto faces = find_faces(vertices, adj);
    for (auto it : faces) {
        for (int i : it) {
            cout << i << " ";
        }
        cout << "\n";
    }
    exit(0);
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
#endif

    init();

    // foo();

    int t;
    cin >> t;
    while (t--) {
        int n, m, k;
        cin >> n >> m >> k;
        vector<pair<int, int>> edges;
        for (int i = 1; i <= m; i++) {
            int u, v;
            cin >> u >> v;
            edges.emplace_back(u, v);
        }

        auto calc = [&](int x) -> Mint {
            // cout << x << endl;
            vector<Mint> f(x), g(x);
            f[0] = k;
            g[0] = 0;
            for (int i = 1; i < x; i++) {
                f[i] = g[i - 1];
                g[i] = f[i - 1] * (k - 1) + g[i - 1] * (k - 2);
            }
            return g[x - 1];
        };
        if (m == 0) {
            cout << calc(n) << "\n";
        } else {
            vector<int> all;
            for (int i = 0; i < m; i++) {
                auto [u, v] = edges[i];
                all.push_back(u);
                all.push_back(v);
            }

            for (int i = 1; i <= n; i++) {
                all.push_back(i);
            }

            sort(all.begin(), all.end());
            all.erase(unique(all.begin(), all.end()), all.end());

            vector<Point> vertices;
            for (int i = 0; i < all.size(); i++) {
                ld x = cos(2 * pi * (all[i] - 1) / n);
                ld y = sin(2 * pi * (all[i] - 1) / n);
                ll x_int = round(x * mult_const);
                ll y_int = round(y * mult_const);
                vertices.emplace_back(x_int, y_int);
            }
            vector<vector<size_t>> adj(vertices.size());
            for (int i = 0; i < all.size(); i++) {
                int j = i + 1 == all.size() ? 0 : i + 1;
                adj[i].push_back(j);
                adj[j].push_back(i);
            }
            for (auto [u, v] : edges) {
                int ui = lower_bound(all.begin(), all.end(), u) - all.begin();
                int vi = lower_bound(all.begin(), all.end(), v) - all.begin();
                adj[ui].push_back(vi);
                adj[vi].push_back(ui);
            }

//            for (int i = 0; i < adj.size(); i++) {
//                cout << i << " : ";
//                for (int j : adj[i]) {
//                    cout << j << " ";
//                }
//                cout << "\n";
//            }

            auto faces = find_faces(vertices, adj);

            Mint ans = 1;

            for (const auto& face : faces) {
                if (face.size() != n) {
                    ans *= calc(face.size());
                }
            }

//            ans = calc(3) * calc(4) * calc(4) * calc(3) * calc(3);
//            ans *= calc(edges[0].first + n - edges[0].second);
//            for (int i = 0; i + 1 < m; i++) {
//                int j1 = edges[i].first;
//                int j2 = edges[i + 1].first;
//                int j3 = edges[i + 1].second;
//                int j4 = edges[i].second;
//                ans *= calc(j2 - j1 + 1 + j4 - j3 + 1);
//            }
//            ans *= calc(edges[m - 1].second - edges[m - 1].first);

            ans /= binpow(Mint(k) * (Mint(k - 1)), m);
            cout << ans << "\n";
        }
    }

#ifdef LOCAL
    cout << "\nTime elapsed: " << double(clock()) / CLOCKS_PER_SEC << " s.\n";
#endif
}