#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

template<int mod>
class Modular {
public:
    int val;
    Modular() : val(0) {}
    Modular(int new_val) : val(new_val) {}
    // Modular(long long new_val) : val(new_val % mod) {}  // AFFECTS OPERATOR* (because it has one more %)
    friend Modular operator+(const Modular& a, const Modular& b) {
        if (a.val + b.val >= mod) return a.val + b.val - mod;
        else return a.val + b.val;
    }
    friend Modular operator-(const Modular& a, const Modular& b) {
        if (a.val - b.val < 0) return a.val - b.val + mod;
        else return a.val - b.val;
    }
    friend Modular operator*(const Modular& a, const Modular& b) {
        return 1ll * a.val * b.val % mod;
    }
    friend Modular binpow(Modular a, long long n) {
        Modular res = 1;
        for (; n; n >>= 1) {
            if (n & 1) res *= a;
            a *= a;
        }
        return res;
    }
    /* ALTERNATIVE INVERSE FUNCTION USING EXTENDED EUCLIDEAN ALGORITHM
    friend void gcd(int a, int b, Modular& x, Modular& y) {
        if (a == 0) {
            x = Modular(0);
            y = Modular(1);
            return;
        }
        Modular x1, y1;
        gcd(b % a, a, x1, y1);
        x = y1 - (b / a) * x1;
        y = x1;
    }
    friend Modular inv(const Modular& a) {
        Modular x, y;
        gcd(a.val, mod, x, y);
        return x;
    }
    */
    friend Modular inv(const Modular& a) {
        return binpow(a, mod - 2);
    }
    Modular operator/(const Modular& ot) const {
        return *this * inv(ot);
    }
    Modular& operator++() {
        if (val + 1 == mod) val = 0;
        else ++val;
        return *this;
    }
    Modular operator++(int) {
        Modular tmp = *this;
        ++(*this);
        return tmp;
    }
    Modular operator+() const {
        return *this;
    }
    Modular operator-() const {
        return 0 - *this;
    }
    Modular& operator+=(const Modular& ot) {
        return *this = *this + ot;
    }
    Modular& operator-=(const Modular& ot) {
        return *this = *this - ot;
    }
    Modular& operator*=(const Modular& ot) {
        return *this = *this * ot;
    }
    Modular& operator/=(const Modular& ot) {
        return *this = *this / ot;
    }
    bool operator==(const Modular& ot) const {
        return val == ot.val;
    }
    bool operator!=(const Modular& ot) const {
        return val != ot.val;
    }
    bool operator<(const Modular& ot) const {
        return val < ot.val;
    }
    bool operator>(const Modular& ot) const {
        return val > ot.val;
    }
    explicit operator int() const {
        return val;
    }
};

template<int mod>
istream& operator>>(istream& istr, Modular<mod>& x) {
    return istr >> x.val;
}

template<int mod>
ostream& operator<<(ostream& ostr, const Modular<mod>& x) {
    return ostr << x.val;
}

const int mod = 998244353; // 998244353
using Mint = Modular<mod>;

const int N = 1e5 + 10;
Mint dp[2][4][N], f[N], g[N];

int x[2][N];

Mint solve(int n) {
    for (int i = 1; i <= n; i++) {
        f[i] = g[i - 1] + 1 + f[i - 1];
        g[i] = g[i - 1] + 1 + g[i - 1] + 1 + g[i - 1];
    }
    for (int s = 0; s <= 1; s++) {
        for (int i = 1; i <= n; i++) {
            dp[s][x[s][i] - 1][i] = dp[s][x[s][i - 1] - 1][i - 1];
            if (x[s][i] == 1) {
                dp[s][1][i] = dp[s][2][i - 1] + 1 + f[i - 1];
                dp[s][2][i] = dp[s][2][i - 1] + 1 + g[i - 1] + 1 + g[i - 1];
            } else if (x[s][i] == 2) {
                dp[s][0][i] = dp[s][2][i - 1] + 1 + g[i - 1];
                dp[s][2][i] = dp[s][0][i - 1] + 1 + g[i - 1];
            } else {
                dp[s][0][i] = dp[s][0][i - 1] + 1 + g[i - 1] + 1 + g[i - 1];
                dp[s][1][i] = dp[s][0][i - 1] + 1 + f[i - 1];
            }
        }
    }
    int ptr = n;
    while (ptr >= 1 && x[0][ptr] == x[1][ptr]) {
        ptr--;
    }
    if (ptr == 0) {
        return 0;
    } else {
        if (x[0][ptr] == 1) {
            if (x[1][ptr] == 2) {
                return dp[0][2][ptr - 1] + 1 + dp[1][2][ptr - 1];
            } else if (x[1][ptr] == 3) {
                return dp[0][2][ptr - 1] + 1 + g[ptr - 1] + 1 + dp[1][0][ptr - 1];
            }
        } else if (x[0][ptr] == 2) {
            if (x[1][ptr] == 1) {
                return dp[0][2][ptr - 1] + 1 + dp[1][2][ptr - 1];
            } else if (x[1][ptr] == 3) {
                return dp[0][0][ptr - 1] + 1 + dp[1][0][ptr - 1];
            }
        } else if (x[0][ptr] == 3) {
            if (x[1][ptr] == 1) {
                return dp[0][0][ptr - 1] + 1 + g[ptr - 1] + 1 + dp[1][2][ptr - 1];
            } else if (x[1][ptr] == 2) {
                return dp[0][0][ptr - 1] + 1 + dp[1][0][ptr - 1];
            }
        }
    }
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
#endif

    int t;
    cin >> t;
    while (t--) {
        int n;
        cin >> n;
        for (int s = 0; s <= 1; s++) {
            for (int i = 1; i <= n; i++) {
                cin >> x[s][i];
            }
        }
        cout << solve(n) << "\n";
    }

#ifdef LOCAL
    cout << "\nTime elapsed: " << double(clock()) / CLOCKS_PER_SEC << " s.\n";
#endif
}