#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

template<int mod>
class Modular {
public:
    int val;
    Modular() : val(0) {}
    Modular(int new_val) : val(new_val) {}
    // Modular(long long new_val) : val(new_val % mod) {}  // AFFECTS OPERATOR* (because it has one more %)
    friend Modular operator+(const Modular& a, const Modular& b) {
        if (a.val + b.val >= mod) return a.val + b.val - mod;
        else return a.val + b.val;
    }
    friend Modular operator-(const Modular& a, const Modular& b) {
        if (a.val - b.val < 0) return a.val - b.val + mod;
        else return a.val - b.val;
    }
    friend Modular operator*(const Modular& a, const Modular& b) {
        return 1ll * a.val * b.val % mod;
    }
    friend Modular binpow(Modular a, long long n) {
        Modular res = 1;
        for (; n; n >>= 1) {
            if (n & 1) res *= a;
            a *= a;
        }
        return res;
    }
    /* ALTERNATIVE INVERSE FUNCTION USING EXTENDED EUCLIDEAN ALGORITHM
    friend void gcd(int a, int b, Modular& x, Modular& y) {
        if (a == 0) {
            x = Modular(0);
            y = Modular(1);
            return;
        }
        Modular x1, y1;
        gcd(b % a, a, x1, y1);
        x = y1 - (b / a) * x1;
        y = x1;
    }
    friend Modular inv(const Modular& a) {
        Modular x, y;
        gcd(a.val, mod, x, y);
        return x;
    }
    */
    friend Modular inv(const Modular& a) {
        return binpow(a, mod - 2);
    }
    Modular operator/(const Modular& ot) const {
        return *this * inv(ot);
    }
    Modular& operator++() {
        if (val + 1 == mod) val = 0;
        else ++val;
        return *this;
    }
    Modular operator++(int) {
        Modular tmp = *this;
        ++(*this);
        return tmp;
    }
    Modular operator+() const {
        return *this;
    }
    Modular operator-() const {
        return 0 - *this;
    }
    Modular& operator+=(const Modular& ot) {
        return *this = *this + ot;
    }
    Modular& operator-=(const Modular& ot) {
        return *this = *this - ot;
    }
    Modular& operator*=(const Modular& ot) {
        return *this = *this * ot;
    }
    Modular& operator/=(const Modular& ot) {
        return *this = *this / ot;
    }
    bool operator==(const Modular& ot) const {
        return val == ot.val;
    }
    bool operator!=(const Modular& ot) const {
        return val != ot.val;
    }
    bool operator<(const Modular& ot) const {
        return val < ot.val;
    }
    bool operator>(const Modular& ot) const {
        return val > ot.val;
    }
    explicit operator int() const {
        return val;
    }
};

template<int mod>
istream& operator>>(istream& istr, Modular<mod>& x) {
    return istr >> x.val;
}

template<int mod>
ostream& operator<<(ostream& ostr, const Modular<mod>& x) {
    return ostr << x.val;
}

const int mod = 998244353; // 998244353
using Mint = Modular<mod>;

const int N = 100 + 10;
Mint dp[N][(1 << 3)][(1 << 16)];
Mint ans[N][(1 << 16)];
Mint F[(1 << 16)];

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
#endif

    int n, m, k;
    cin >> n >> m >> k;
    string w;
    cin >> w;

    int mask_bits = (1 << (k + 1));
    for (int mask = 0; mask < (1 << (k + 1)); mask++) {
        dp[k + 1][mask >> 1][1 << mask]++;
    }
    for (int i = k + 1; i + 1 <= N - 1; i++) {
        for (int suf = 0; suf < (1 << k); suf++) {
            for (int mask = 0; mask < (1 << mask_bits); mask++) {
                for (int x = 0; x <= 1; x++) {
                    int nxt = suf + (x == 0 ? (1 << k) : 0);
                    dp[i + 1][nxt >> 1][mask | (1 << nxt)] += dp[i][suf][mask];
                }
            }
        }
    }
    for (int i = k + 1; i <= N - 1; i++) {
        for (int suf = 0; suf < (1 << k); suf++) {
            for (int mask = 0; mask < (1 << mask_bits); mask++) {
                ans[i][mask] += dp[i][suf][mask];
            }
        }
    }
    int good_masks = 0;
    for (int x = 0; x < (1 << (k + 1)); x++) {
        string cur(k + 1, '0');
        for (int i = 0; i < k + 1; i++) {
            if (x & (1 << i)) {
                cur[i] = '1';
            }
        }
        if (cur.substr(0, k) == w || cur.substr(1, k) == w) {
            good_masks |= (1 << x);
        }
    }
    for (int i = k + 1; i <= N - 1; i++) {
        for (int mask = 0; mask < (1 << mask_bits); mask++) {
            if ((mask & good_masks) == 0) {
                ans[i][mask] = 0;
            }
        }
    }

    if (n == k) {
        if (m == k) {
            cout << "1\n";
        } else {
            Mint res = 0;
            for (int mask = 0; mask < (1 << mask_bits); mask++) {
                res += ans[m][mask];
            }
            cout << res << "\n";
        }
    } else {
        if (m == k) {
            Mint res = 0;
            for (int mask = 0; mask < (1 << mask_bits); mask++) {
                res += ans[n][mask];
            }
            cout << res << "\n";
        } else {
            for(int i = 0; i<(1<<mask_bits); ++i)
                F[i] = ans[m][i];
            for(int i = 0;i < mask_bits; ++i) for(int mask = 0; mask < (1<<mask_bits); ++mask){
                    if(mask & (1<<i))
                        F[mask] += F[mask^(1<<i)];
                }

            Mint res = 0;
            int all = (1 << mask_bits) - 1;
            for (int mask = 0; mask < (1 << mask_bits); mask++) {
                res += ans[n][mask] * F[all ^ mask];
            }
            cout << res << "\n";
        }
    }

#ifdef LOCAL
    cout << "\nTime elapsed: " << double(clock()) / CLOCKS_PER_SEC << " s.\n";
#endif
}