from collections import deque

def min_wall_length(test_cases):
    results = []
    
    for case in test_cases:
        n, m, maze = case
        start, end = None, None
        
        # Поиск входа (s) и выхода (f)
        for i in range(n):
            for j in range(m):
                if maze[i][j] == 's':
                    start = (i, j)
                elif maze[i][j] == 'f':
                    end = (i, j)
        
        if not start or not end:
            results.append(-1)
            continue

        # BFS для проверки достижимости выхода из входа
        directions = [(-1, 0), (1, 0), (0, -1), (0, 1)]
        visited = [[False] * m for _ in range(n)]
        queue = deque([start])
        visited[start[0]][start[1]] = True
        reachable = False

        while queue:
            x, y = queue.popleft()
            if (x, y) == end:
                reachable = True
                break
            for dx, dy in directions:
                nx, ny = x + dx, y + dy
                if 0 <= nx < n and 0 <= ny < m and not visited[nx][ny] and maze[nx][ny] != '#':
                    visited[nx][ny] = True
                    queue.append((nx, ny))
        
        if not reachable:
            results.append(-1)
            continue

        # Определение минимальной стены
        min_wall = float('inf')

        for i in range(n):  # Горизонтальные стены
            for j in range(m - 1):
                if maze[i][j] == '.' and maze[i][j + 1] == '.':
                    wall = [(i, k) for k in range(j, j + 2)]
                    if check_wall(wall, start, end, n, m, maze):
                        min_wall = min(min_wall, len(wall))
        
        for j in range(m):  # Вертикальные стены
            for i in range(n - 1):
                if maze[i][j] == '.' and maze[i + 1][j] == '.':
                    wall = [(k, j) for k in range(i, i + 2)]
                    if check_wall(wall, start, end, n, m, maze):
                        min_wall = min(min_wall, len(wall))
        
        results.append(min_wall if min_wall != float('inf') else -1)
    
    return results

def check_wall(wall, start, end, n, m, maze):
    # Создание копии лабиринта с установленной стеной
    temp_maze = [list(row) for row in maze]
    for x, y in wall:
        temp_maze[x][y] = '#'
    
    # BFS для проверки достижимости выхода после установки стены
    directions = [(-1, 0), (1, 0), (0, -1), (0, 1)]
    visited = [[False] * m for _ in range(n)]
    queue = deque([start])
    visited[start[0]][start[1]] = True

    while queue:
        x, y = queue.popleft()
        if (x, y) == end:
            return False
        for dx, dy in directions:
            nx, ny = x + dx, y + dy
            if 0 <= nx < n and 0 <= ny < m and not visited[nx][ny] and temp_maze[nx][ny] != '#':
                visited[nx][ny] = True
                queue.append((nx, ny))
    
    return True

# Чтение входных данных
t = int(input())
test_cases = []
for _ in range(t):
    n, m = map(int, input().split())
    maze = [input().strip() for _ in range(n)]
    test_cases.append((n, m, maze))

# Вывод результатов
results = min_wall_length(test_cases)
for res in results:
    print(res)
