import java.util.*;
import java.io.*;

public class Main {

    private static final int MOD = 998244353;

    // Function to compute a^b % MOD using binary exponentiation
    public static long modExp(long a, long b, long mod) {
        long result = 1;
        a = a % mod;
        while (b > 0) {
            if ((b & 1) == 1) {
                result = (result * a) % mod;
            }
            a = (a * a) % mod;
            b >>= 1;
        }
        return result;
    }

    // Solve the problem for each test case
    public static void solveTestCase(int n, int m, int k, List<int[]> cuts) {
        // If no cuts, the cake is just a cycle, calculate the cycle coloring
        if (m == 0) {
            // Coloring the cycle with no two adjacent same colors
            long result = (k * modExp(k - 1, n - 1, MOD)) % MOD;
            System.out.println(result);
            return;
        }

        // Union-Find (Disjoint Set Union) to track the connected components
        int[] parent = new int[n];
        for (int i = 0; i < n; i++) {
            parent[i] = i;
        }

        // Find and union operations
        int find(int x) {
            if (parent[x] != x) {
                parent[x] = find(parent[x]);
            }
            return parent[x];
        }

        void union(int x, int y) {
            int rootX = find(x);
            int rootY = find(y);
            if (rootX != rootY) {
                parent[rootX] = rootY;
            }
        }

        // Apply the cuts (union corners connected by cuts)
        for (int[] cut : cuts) {
            int u = cut[0] - 1;  // 0-based index
            int v = cut[1] - 1;  // 0-based index
            union(u, v);
        }

        // Count the number of connected components
        Set<Integer> components = new HashSet<>();
        for (int i = 0; i < n; i++) {
            components.add(find(i));
        }

        // If there is only one component, this is equivalent to coloring a cycle
        if (components.size() == 1) {
            long result = (k * modExp(k - 1, n - 1, MOD)) % MOD;
            System.out.println(result);
        } else {
            // Otherwise, the coloring must be computed based on the connected components
            // Each component behaves like a cycle, and we must calculate the valid colorings
            long result = modExp(k, components.size(), MOD);
            System.out.println(result);
        }
    }

    public static void main(String[] args) throws IOException {
        Scanner sc = new Scanner(System.in);
        int t = sc.nextInt();  // Number of test cases
        
        // Process each test case
        for (int caseNum = 0; caseNum < t; caseNum++) {
            int n = sc.nextInt();  // Number of cake corners
            int m = sc.nextInt();  // Number of cuts
            int k = sc.nextInt();  // Number of available colors
            
            List<int[]> cuts = new ArrayList<>();
            for (int i = 0; i < m; i++) {
                int u = sc.nextInt();
                int v = sc.nextInt();
                cuts.add(new int[] { u, v });
            }
            
            solveTestCase(n, m, k, cuts);
        }
        
        sc.close();
    }
}