from collections import deque

def bfs(maze, start, n, m):
    queue = deque([start])
    visited = set()
    visited.add(start)
    
    directions = [(-1, 0), (1, 0), (0, -1), (0, 1)]  # Up, Down, Left, Right
    
    while queue:
        x, y = queue.popleft()
        
        for dx, dy in directions:
            nx, ny = x + dx, y + dy
            
            if 0 <= nx < n and 0 <= ny < m and maze[nx][ny] != '#' and (nx, ny) not in visited:
                visited.add((nx, ny))
                queue.append((nx, ny))
    
    return visited

def minimum_wall_length(n, m, maze):
    # Find the positions of 's' and 'f'
    start = end = None
    for i in range(n):
        for j in range(m):
            if maze[i][j] == 's':
                start = (i, j)
            elif maze[i][j] == 'f':
                end = (i, j)
    
    # BFS to find all reachable cells from 's'
    reachable_from_s = bfs(maze, start, n, m)
    
    # If 'f' is not reachable, no wall is needed
    if end not in reachable_from_s:
        return -1  # Shouldn't happen as per constraints
    
    # Check every row and column for the minimum blocking wall
    min_wall_length = float('inf')
    
    # Check rows
    for i in range(n):
        wall_length = 0
        for j in range(m):
            if (i, j) in reachable_from_s:
                wall_length += 1
            else:
                if wall_length > 0:
                    min_wall_length = min(min_wall_length, wall_length)
                wall_length = 0
        if wall_length > 0:
            min_wall_length = min(min_wall_length, wall_length)
    
    # Check columns
    for j in range(m):
        wall_length = 0
        for i in range(n):
            if (i, j) in reachable_from_s:
                wall_length += 1
            else:
                if wall_length > 0:
                    min_wall_length = min(min_wall_length, wall_length)
                wall_length = 0
        if wall_length > 0:
            min_wall_length = min(min_wall_length, wall_length)
    
    return min_wall_length if min_wall_length != float('inf') else -1

# Input
t = int(input())  # Number of test cases
results = []

for _ in range(t):
    n, m = map(int, input().split())
    maze = [input().strip() for _ in range(n)]
    results.append(minimum_wall_length(n, m, maze))

# Output results
for res in results:
    print(res)
