import java.util.*;

public class MazeWallBlock {
    static final int[] dirX = {-1, 1, 0, 0}; // Directions: up, down, left, right
    static final int[] dirY = {0, 0, -1, 1};
    
    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);
        int t = sc.nextInt(); // Number of test cases
        sc.nextLine(); // Consume newline
        
        while (t-- > 0) {
            int n = sc.nextInt();
            int m = sc.nextInt();
            sc.nextLine(); // Consume newline
            
            char[][] maze = new char[n][m];
            int sx = -1, sy = -1, fx = -1, fy = -1;
            
            // Read the maze and find the entrance (s) and exit (f)
            for (int i = 0; i < n; i++) {
                String row = sc.nextLine();
                for (int j = 0; j < m; j++) {
                    maze[i][j] = row.charAt(j);
                    if (maze[i][j] == 's') {
                        sx = i; sy = j;
                    } else if (maze[i][j] == 'f') {
                        fx = i; fy = j;
                    }
                }
            }
            
            // BFS from entrance 's' to find the shortest path to exit 'f'
            int[][] dist = new int[n][m];
            for (int i = 0; i < n; i++) {
                Arrays.fill(dist[i], -1); // Initialize distances as unvisited
            }
            Queue<int[]> queue = new LinkedList<>();
            queue.offer(new int[]{sx, sy});
            dist[sx][sy] = 0;
            
            while (!queue.isEmpty()) {
                int[] current = queue.poll();
                int x = current[0], y = current[1];
                
                for (int i = 0; i < 4; i++) {
                    int nx = x + dirX[i], ny = y + dirY[i];
                    if (nx >= 0 && nx < n && ny >= 0 && ny < m && maze[nx][ny] != '#' && dist[nx][ny] == -1) {
                        dist[nx][ny] = dist[x][y] + 1;
                        queue.offer(new int[]{nx, ny});
                    }
                }
            }
            
            // If the exit is not reachable, return -1
            if (dist[fx][fy] == -1) {
                System.out.println(-1);
                continue;
            }
            
            // Now check if we can block the path by placing a horizontal or vertical wall
            int minWallLength = Integer.MAX_VALUE;
            
            // Try horizontal walls: Block entire rows
            for (int i = 0; i < n; i++) {
                boolean canBlock = true;
                for (int j = 0; j < m; j++) {
                    if (maze[i][j] == 's' || maze[i][j] == 'f' || maze[i][j] == '#') {
                        continue;
                    
                    