import java.util.*;
import java.io.*;

public class CapybaraCake {
    static final int MOD = 998244353;

    // Disjoint Set Union (DSU) class to handle union-find operations
    static class DSU {
        int[] parent, size;
        
        DSU(int n) {
            parent = new int[n + 1];
            size = new int[n + 1];
            for (int i = 1; i <= n; i++) {
                parent[i] = i;
                size[i] = 1;
            }
        }
        
        // Find the representative of the set containing x
        int find(int x) {
            if (parent[x] != x) {
                parent[x] = find(parent[x]);
            }
            return parent[x];
        }

        // Union operation to merge the sets containing x and y
        void union(int x, int y) {
            int rootX = find(x);
            int rootY = find(y);
            if (rootX != rootY) {
                if (size[rootX] < size[rootY]) {
                    int temp = rootX;
                    rootX = rootY;
                    rootY = temp;
                }
                parent[rootY] = rootX;
                size[rootX] += size[rootY];
            }
        }
    }

    // Fast modular exponentiation
    static long modPow(long base, long exp, int mod) {
        long result = 1;
        while (exp > 0) {
            if (exp % 2 == 1) {
                result = (result * base) % mod;
            }
            base = (base * base) % mod;
            exp /= 2;
        }
        return result;
    }

    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);
        int t = sc.nextInt();  // Number of test cases
        StringBuilder sb = new StringBuilder();

        while (t-- > 0) {
            int n = sc.nextInt();  // Number of cake corners
            int m = sc.nextInt();  // Number of cuts
            int k = sc.nextInt();  // Number of available colors
            
            DSU dsu = new DSU(n);
            
            // Process cuts (edges in the graph)
            for (int i = 0; i < m; i++) {
                int u = sc.nextInt();
                int v = sc.nextInt();
                dsu.union(u, v);
            }

            // Find number of connected components
            Set<Integer> uniqueRoots = new HashSet<>();
            for (int i = 1; i <= n; i++) {
                uniqueRoots.add(dsu.find(i));
            }

            // Calculate ways to color the connected components
            long result = 1;
            for (int component : uniqueRoots) {
                int size = dsu.size[component];

                if (size == 1) {
                    // A single node, just k ways to color it
                    result = (result * k) % MOD;
                } else if (size == 2) {
                    // A 2-node chain, k * (k-1) ways
                    result = (result * k * (k - 1)) % MOD;
                } else {
                    // A cycle, k * (k-1)^(size-1) ways
                    result = (result * k % MOD) * modPow(k - 1, size - 1, MOD) % MOD;
                }
            }

            sb.append(result).append("\n");
        }
        
        System.out.print(sb);
    }
}
