import java.util.*;
import java.io.*;

public class Main {
    static class Circle {
        long x, y, r;
        Circle(long x, long y, long r) {
            this.x = x;
            this.y = y;
            this.r = r;
        }
    }

    public static void main(String[] args) throws IOException {
        // Using buffered IO for fast input/output
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        StringBuilder sb = new StringBuilder();

        // Reading number of test cases
        int t = Integer.parseInt(br.readLine());

        // Processing each test case
        while (t-- > 0) {
            // Read the number of circles for this test case
            int n = Integer.parseInt(br.readLine());
            Circle[] circles = new Circle[n];

            // Read the details of the circles
            for (int i = 0; i < n; i++) {
                String[] parts = br.readLine().split(" ");
                long x = Long.parseLong(parts[0]);
                long y = Long.parseLong(parts[1]);
                long r = Long.parseLong(parts[2]);
                circles[i] = new Circle(x, y, r);
            }

            // Count the number of valid 8-shaped figure pairs
            int count = count8ShapedPairs(circles);
            sb.append(count).append("\n");
        }

        // Output the result for all test cases
        System.out.print(sb.toString());
    }

    // Function to count 8-shaped pairs
    public static int count8ShapedPairs(Circle[] circles) {
        int count = 0;
        int n = circles.length;

        // Sort circles by their x-coordinate and then by radius (to optimize comparisons)
        Arrays.sort(circles, (c1, c2) -> {
            if (c1.x != c2.x) return Long.compare(c1.x, c2.x);
            return Long.compare(c1.r, c2.r);
        });

        // Compare every pair of circles
        for (int i = 0; i < n; i++) {
            Circle c1 = circles[i];
            for (int j = i + 1; j < n; j++) {
                Circle c2 = circles[j];
                
                // Calculate the distance between centers
                long dx = c2.x - c1.x;
                long dy = c2.y - c1.y;
                long distanceSquared = dx * dx + dy * dy;
                long sumRadii = c1.r + c2.r;

                // Check if the circles touch externally (distance = sum of radii)
                if (distanceSquared == sumRadii * sumRadii) {
                    count++;
                }
            }
        }

        return count;
    }
}
